@@ -2,7 +2,7 @@
 ### D E F A U L T   P A R   V A L U E S   &   P R E S E T S
 ###
 
-# devtools::use_data(permittedValues, defaults, presets, overwrite = TRUE)
+# usethis::use_data(permittedValues, defaults, presets, overwrite = TRUE)
 
 
 #' Defaults and ranges
@@ -37,10 +37,10 @@ permittedValues = matrix(c(
   'shimmerDep', 0, 0, 100, 1,
   'shimmerLen', 1, 1, 100, 1,
   'attackLen', 50, 0, 200, 10,
+  'glottisAnchors', 0, 0, 5000, 5,
+  'glottis', 0, 0, 5000, 5,
   'rolloff', -9, -30, 0, 1,
-  'glottisAnchors', 0, 0, 1000, 5,
-  'glottis', 0, 0, 1000, 5,
-  'rolloffOct', -3, -30, 10, 1,
+  'rolloffOct', 0, -30, 3, 1,
   'rolloffParab', 0, -30, 30, 1,
   'rolloffParabHarm', 3, 1, 20, 1,
   'rolloffKHz', -3, -20, 0, 1,
@@ -85,7 +85,7 @@ temp = permittedValues[,1]
 permittedValues = apply (permittedValues[,2:5], 2, as.numeric)
 colnames(permittedValues) = c('default', 'low', 'high', 'step')
 rownames(permittedValues) = temp
-# devtools::use_data(permittedValues, overwrite = TRUE)
+# usethis::use_data(permittedValues, overwrite = TRUE)
 
 # a list of default values for Shiny app - mostly the same as for
 # soundgen(). NB: if defaults change, this has to be updated!!!
@@ -107,7 +107,7 @@ defaults = list(
   shimmerLen = 1,
   attackLen = 50,
   rolloff = -9,
-  rolloffOct = -3,
+  rolloffOct = 0,
   rolloffParab = 0,
   rolloffParabHarm = 3,
   rolloffKHz = -3,
@@ -171,7 +171,7 @@ defaults = list(
   glottisAnchorsDep = .1,
   specDep = .1
 )
-# devtools::use_data(defaults, overwrite = TRUE)
+# usethis::use_data(defaults, overwrite = TRUE)
 
 # -------------------------------------------------------------
 # A library of presets for easy generation of a few nice sounds
@@ -186,17 +186,17 @@ presets = list(
   M1 = list(
     Vowel1 = 'soundgen()', # these are just the global defaults
 
-    Gasp = 'soundgen(sylLen = 240, pitch = c(160, 150), nonlinBalance = 100, jitterDep = 0.6, rolloff = -24, rolloffOct = -7, rolloffParab = 15, rolloffParabHarm = 2, formantDep = 1.5, subDep = 0, shortestEpoch = 100, noise = list(time = c(-1, 170, 362), value = c(-44, -32, -56)), rolloffNoise = -7, mouth = list(time = c(0, 0.07, 1), value = c(0, 0.48, 0.32)))',
+    Gasp = 'soundgen(sylLen = 240, pitch = c(160, 150), nonlinBalance = 100, jitterDep = 0.6, rolloff = -24, rolloffParab = 15, rolloffParabHarm = 2, formantDep = 1.5, subDep = 0, shortestEpoch = 100, noise = list(time = c(-1, 170, 362), value = c(-44, -32, -56)), rolloffNoise = -7, mouth = list(time = c(0, 0.07, 1), value = c(0, 0.48, 0.32)))',
 
-    Roar = 'soundgen(sylLen = 960, pitch = list(time = c(0, 0.13, 0.9, 1), value = c(151, 187, 139, 120)), temperature = 0.1, nonlinBalance = 60, jitterDep = 0.7, rolloff = -12, rolloffOct = 3, formants = c(500, 1000, 1500), vocalTract = 19, subFreq = 90, subDep = 60, shortestEpoch = 150, mouth = list(time = c(0, 0.1, 1), value = c(0.38, 0.71, 0.42)))',
+    Roar = 'soundgen(sylLen = 960, pitch = list(time = c(0, 0.13, 0.9, 1), value = c(151, 187, 139, 120)), temperature = 0.1, nonlinBalance = 60, jitterDep = 0.7, rolloff = -12, rolloffOct = 1, formants = c(500, 1000, 1500), vocalTract = 19, subFreq = 90, subDep = 60, shortestEpoch = 150, mouth = list(time = c(0, 0.1, 1), value = c(0.38, 0.71, 0.42)))',
 
-    Moan = 'soundgen(sylLen = 800, pitch = list(time = c(0, 0.21, 1), value = c(230, 181, 143)), nonlinBalance = 100, jitterDep = 1, jitterLen = 60, attackLen = 100, rolloff = -12, rolloffOct = 0, formants = list(f1 = c(630, 860), f2 = c(900, 1430), f3 = c(3000, 2900), f4 = c(3960, 3960)), subDep = 0, noise = list(time = c(-20, 801, 911), value = c(-39, -27, -80)), rolloffNoise = -8, mouth = c(0.54, 0.3), ampl = c(0, -9))',
+    Moan = 'soundgen(sylLen = 800, pitch = list(time = c(0, 0.21, 1), value = c(230, 181, 143)), nonlinBalance = 100, jitterDep = 1, jitterLen = 60, attackLen = 100, rolloff = -12, formants = list(f1 = c(630, 860), f2 = c(900, 1430), f3 = c(3000, 2900), f4 = c(3960, 3960)), subDep = 0, noise = list(time = c(-20, 801, 911), value = c(-39, -27, -80)), rolloffNoise = -8, mouth = c(0.54, 0.3), ampl = c(0, -9))',
 
     Sigh = 'soundgen(sylLen = 50, pitch = NA, temperature = 0.1, formants = c(860, 1430, 2900), vocalTract = 17, noise = list(time = c(-20, 104, 756, 1252), value = c(26, 40, 23, -22)))',
 
     Laugh = 'soundgen(nSyl = 5, sylLen = 120, pauseLen = 120, pitch = c(180, 166), pitchGlobal = list(time = c(0, 0.3, 1), value = c(1, 2, 0)), temperature = 0.1, nonlinBalance = 50, jitterDep = 0.8, attackLen = 10, formants = list(f1 = c(860, 550, 550), f2 = c(1430, 2300, 2300), f3 = c(2900, 2800, 2800), f4 = 4000), subDep = 0, shortestEpoch = 50, noise = list(time = c(-10, 57, 237), value = c(-80, -40, -80)), rolloffNoise = -6, ampl = list(time = c(0, 0.32, 1), value = c(0, -20, 0)), amplGlobal = list(time = c(0, 0.77, 1), value = c(0, -11, -40)))',
 
-    Snore = 'soundgen(sylLen = 960, pitch = list(time = c(0, 0.15, 0.87, 1), value = c(175, 199, 188, 140)), temperature = 0.1, nonlinBalance = 100, jitterDep = 0.75, rolloffOct = -14, formants = c(560, 1000, 1450, 3800), subDep = 80, shortestEpoch = 200, noise = list(time = c(-18, 991), value = c(-48, -48)), mouth = 0)',
+    Snore = 'soundgen(sylLen = 960, pitch = list(time = c(0, 0.15, 0.87, 1), value = c(175, 199, 188, 140)), temperature = 0.1, nonlinBalance = 100, jitterDep = 0.75, rolloffOct = -5, formants = c(560, 1000, 1450, 3800), subDep = 80, shortestEpoch = 200, noise = list(time = c(-18, 991), value = c(-48, -48)), mouth = 0)',
 
     # 'Formants' is a reserved name. The list of presets for every caller should end with
     # a list of 'Formants' presets for each vowel and consonant, otherwise you won't be
@@ -312,7 +312,7 @@ presets = list(
   Chimpanzee = list(
     Bark_alarm = 'soundgen(sylLen = 160, pitch = c(232, 185), nonlinBalance = 100, jitterDep = 2.8, attackLen = 61, rolloff = -19, formants = c(400, 1000), vocalTract = 14, subFreq = 125, subDep = 60, noise = list(time = c(0, 63, 344), value = c(-80, -10, -80)), rolloffNoise = -6)',
 
-    Scream_conflict = 'soundgen(sylLen = 740, pitch = list(time = c(0, 0.3, 0.9, 1), value = c(1200, 1547, 1487, 1154)), formants = NULL, vocalTract = 14, temperature = 0.05, nonlinBalance = 100, jitterDep = 0.3, rolloff = -6, rolloffOct = 0, subFreq = 75, subDep = 130)',
+    Scream_conflict = 'soundgen(sylLen = 740, pitch = list(time = c(0, 0.3, 0.9, 1), value = c(1200, 1547, 1487, 1154)), formants = NULL, vocalTract = 14, temperature = 0.05, nonlinBalance = 100, jitterDep = 0.3, rolloff = -6, subFreq = 75, subDep = 130)',
 
     Grunt_excited = 'soundgen(nSyl = 6, sylLen = 100, pauseLen = 220, pitch = c(216, 164), pitchGlobal = list(time = c(0, 0.4, 1), value = c(0, 1, -8.4)), temperature = 0.15, nonlinBalance = 100, jitterDep = 3.1, attackLen = 10, formants = list(f1 = c(410, 250), f2 = c(990, 650)), vocalTract = 20.5, subDep = 0, noise = list(time = c(-8, 22, 298), value = c(-80, -41, -80)), rolloffNoise = -7, ampl = c(0, -30), amplGlobal = list(time = c(0, 0.35, 1), value = c(-10, 0, -20)))',
 
@@ -326,23 +326,23 @@ presets = list(
   ),
 
   Cat = list(
-    Chirp = 'soundgen(sylLen = 120, pitch = c(828, 768, 423), nonlinBalance = 100, jitterDep = 0.3, shimmerDep = 11, attackLen = 1, rolloff = -3, rolloffOct = -4, formants = c(875, 1944, 2409), subDep = 0, noise = list(time = c(-1, 80, 161), value = c(-60, 14, -47)), ampl = list(time = c(0, 0.33, 1), value = c(0, 0, -11)), windowLength = 10)',
+    Chirp = 'soundgen(sylLen = 120, pitch = c(828, 768, 423), nonlinBalance = 100, jitterDep = 0.3, shimmerDep = 11, attackLen = 1, rolloff = -3, formants = c(875, 1944, 2409), subDep = 0, noise = list(time = c(-1, 80, 161), value = c(-60, 14, -47)), ampl = list(time = c(0, 0.33, 1), value = c(0, 0, -23)))',
 
     Growl = 'soundgen(sylLen = 2700, pitch = c(154, 121), nonlinBalance = 100, jitterDep = 1.3, jitterLen = 6, formants = c(200, 400, 700), vocalTract = 8, subFreq = 80, subDep = 50, shortestEpoch = 50)',
 
     Hiss = 'soundgen(pitch = NULL, formants = c(1112, 2610, 4587), formantDep = 0.8, noise = list(time = c(-185, 74, 506), value = c(-5, 40, 0)), rolloffNoise = -8, mouth = list(time = c(0, 0.13, 1), value = c(0, 0.5, 0.5)))',
 
-    Howl = 'soundgen(sylLen = 3150, pitch = list(time = c(0, 0.05, 0.35, 0.89, 1), value = c(218, 339, 352, 291, 229)), temperature = 0.125, rolloff = -5, rolloffOct = -2, rolloffParabHarm = 1, formants = list(f1 = 895, f2 = c(1500, 2000), f3 = 3287), vocalTract = 10, mouth = list(time = c(0, 0.18, 0.25, 0.41, 0.89, 1), value = c(0.32, 0.24, 0.66, 0.37, 0.39, 0.27)))',
+    Howl = 'soundgen(sylLen = 3150, pitch = list(time = c(0, 0.05, 0.35, 0.89, 1), value = c(218, 339, 352, 291, 229)), temperature = 0.125, rolloff = -5, rolloffParabHarm = 1, formants = list(f1 = 895, f2 = c(1500, 2000), f3 = 3287), vocalTract = 10, mouth = list(time = c(0, 0.18, 0.25, 0.41, 0.89, 1), value = c(0.32, 0.24, 0.66, 0.37, 0.39, 0.27)))',
 
     Heat = 'soundgen(sylLen = 800, pitch = c(480, 552, 501), temperature = 0.1, nonlinBalance = 100, jitterDep = 0.3, jitterLen = 60, rolloff = -6, rolloffParabHarm = 1, formants = list(f1 = c(1500, 500), f2 = 2100, f3 = 5500), subDep = 0, amDep = 20, amFreq = 40, mouth = list(time = c(0, 0.12, 0.86, 1), value = c(0, 0.52, 0.57, 0)))',
 
-    Meow = 'soundgen(sylLen = 920, pitch = list(time = c(0, 0.2, 1), value = c(480, 550, 515)), attackLen = 150, rolloff = -2, rolloffOct = -4, formants = c(1594, 3600), mouth = list(time = c(0, 0.16, 0.91, 1), value = c(0, 1, 0.31, 0.06)))',
+    Meow = 'soundgen(sylLen = 920, pitch = list(time = c(0, 0.2, 1), value = c(480, 550, 515)), attackLen = 150, rolloff = -12, formants = c(1594, 3600), mouth = list(time = c(0, 0.16, 0.91, 1), value = c(0, 1, 0.31, 0.06)))',
 
-    Purr = 'soundgen(repeatBout = 2, nSyl = 2, sylLen = 800, pauseLen = 50, pitch = 25, glottis = 170, temperature = 0.1, jitterDep = 0, jitterLen = 34, rolloff = -30, lipRad = 0, formants = c(1200, 2600, 5200, 7000), subDep = 0, noise = list(time = c(-113, 96, 839), value = c(-80, -39, -58)))',
+    Purr = 'soundgen(repeatBout = 2, nSyl = 2, sylLen = 800, pauseLen = 20, pitch = 25, rolloff = -3, nonlinBalance = 100, subDep = 0, jitterDep = .5, shimmerDep = 20, formants = c(85, 230, 450, 1200, 2600, 5200, 7000), noise = list(time = c(-113, 96, 839), value = c(-80, -40, -70)), formantsNoise = c(1200, 2600, 5200, 7000), rolloffNoise = -3, temperature = 0.15, tempEffects = list(pitchDriftDep = 1))',
 
     Scream = 'soundgen(repeatBout = 2, sylLen = 1610, pauseLen = 500, pitch = list(time = c(0, 0.28, 0.53, 0.88, 1), value = c(388, 385, 669, 663, 392)), temperature = 0.05, nonlinBalance = 35, jitterDep = 1.9, rolloff = -6, formants = list(f1 = c(1800, 1000), f2 = 2400), subFreq = 150, subDep = 80)',
 
-    Snarl = 'soundgen(sylLen = 450, pitch = list(time = c(0, 0.08, 1), value = c(193, 454, 434)), temperature = 0.1, nonlinBalance = 75, jitterDep = 1.1, attackLen = 0, rolloff = -6, rolloffOct = -2, formants = list(f1 = list(freq = c(1500, 2000), width = 300), f2 = list(freq = 2500, width = 350)), subFreq = 230, subDep = 170, shortestEpoch = 75, noise = -8, mouth = list(time = c(0, 0.3, 0.63, 1), value = c(0, 1, 1, 0.5)), windowLength = 10)',
+    Snarl = 'soundgen(sylLen = 450, pitch = list(time = c(0, 0.08, 1), value = c(193, 454, 434)), temperature = 0.1, nonlinBalance = 75, jitterDep = 1.1, attackLen = 0, rolloff = -6, rolloffOct = -1, formants = list(f1 = list(freq = c(1500, 2000), width = 300), f2 = list(freq = 2500, width = 350)), subFreq = 230, subDep = 170, shortestEpoch = 75, noise = -8, mouth = list(time = c(0, 0.3, 0.63, 1), value = c(0, 1, 1, 0.5)))',
 
     Formants = list( # reserved name - the list of presets for every caller must end with a list of 'Formants' presets for each vowel and consonant
       # ...
@@ -350,11 +350,11 @@ presets = list(
   ),
 
   Misc = list(
-    Crocodile = 'soundgen(sylLen = 2570, pitch = list(time = c(0, 0.36, 1), value = c(25, 42, 25)), glottis = c(150, 200), rolloff = -25, formants = c(200, 500, 1200, 1800), temperature = .25, invalidArgAction = "ignore")',
+    Crocodile = 'soundgen(sylLen = 2570, pitch = list(time = c(0, 0.36, 1), value = c(25, 42, 25)), glottis = c(15, 80), rolloff = c(-30, -23, -22, -25), rolloffOct = 1.7, rolloffParab = -20, rolloffParabHarm = 1, nonlinBalance = 100, jitterDep = .5, shimmerDep = 20, formants = c(200, 500, 1200, 1700, 2280, 2680, 3350), formantDep = .8, noise = c(-60, -40, -60), dynamicRange = 120, temperature = .01)',
 
     Dog_bark = 'soundgen(repeatBout = 2, sylLen = 140, pauseLen = 100, pitch = list(time = c(0, 0.29, 1), value = c(559, 716, 647)), temperature = 0.05, nonlinBalance = 100, jitterDep = 2.1, formants = c(1500, 3300, 6000), vocalTract = 8.5, subDep = 0, noise = list(time = c(0, 78, 160), value = c(-80, 12, -80)), rolloffNoise = -9, mouth = c(0, 0.5, 0))',
 
-    Duck = 'soundgen(repeatBout = 5, sylLen = 110, pauseLen = 170, pitch = c(119, 110), temperature = 0.1, rolloff = -3, rolloffOct = -2, formants = c(1600, 2700, 5600, 6400), noise = -13, mouth = c(0.34, 0.57, 0.35))',
+    Duck = 'soundgen(repeatBout = 5, sylLen = 110, pauseLen = 170, pitch = c(119, 110), temperature = 0.1, rolloff = -3, rolloffOct = -1, formants = c(1600, 2700, 5600, 6400), noise = -13, mouth = c(0.34, 0.57, 0.35))',
 
     Elephant = 'soundgen(sylLen = 510, pitch = list(time = c(0, 0.36, 1), value = c(450, 485, 328)), nonlinBalance = 50, jitterDep = 0.3, rolloff = -3, rolloffOct = -2, rolloffKHz = 0, formants = NULL, formantDepStoch = 0, subFreq = 75, subDep = 40, shortestEpoch = 50, noise = list(time = c(0, 510), value = c(-19, -19)), ampl = list(time = c(0, 0.5, 1), value = c(0, 0, -10)))',
 
